package com.ibiz.mybatis.ht;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Collection;
import java.io.Serializable;
import java.lang.reflect.Field;

import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONArray;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Repository;
import org.springframework.util.ObjectUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibiz.core.repository.BCRepository;
import com.ibiz.core.repository.po.BC;
import com.ibiz.core.ht.filter.BCSearchContext;
import com.ibiz.mybatis.ht.mapper.T_BCMapper;
import com.ibiz.mybatis.ht.po.T_BCImpl;
import com.ibiz.mybatis.util.helper.QueryBuilder;
import com.ibiz.mybatis.util.helper.DEHelper;

/**
 * 关系型数据实体表[T_BC]存储对象
 */
@Slf4j
@Repository
@ConditionalOnProperty(prefix = "bc", value="store" , havingValue = "sql", matchIfMissing = true)
public class T_BCRepositoryImpl extends ServiceImpl<T_BCMapper, T_BCImpl> implements BCRepository {

    private Log logger= LogFactory.getLog(T_BCRepositoryImpl.class);


    /**
     * mybatis条件构造器
     */
    @Autowired
    QueryBuilder queryBuilder;
    /**
     * [T_BC]持久层映射对象
     */
    @Autowired
	T_BCMapper t_BCMapper;

    /**
     * 实体辅助对象
     */
    @Autowired
    DEHelper deHelper;

    /**
     * 创建持久层对象
     * @return
     */
    public BC createPO(){
        return new T_BCImpl();
    }

    /**
     * 获取数据
     * @param key 业务数据主键
     * @return
     */
    public BC get(String key){
        return this.getById(key);
    }

    /**
     * 新建数据
     * @param po 持久层数据对象
     * @return
     */
    public boolean create(BC po){
        boolean bOk = this.save((T_BCImpl)po);
        return bOk;
    }

    /**
     * 更新数据
     * @param po 持久层数据对象
     * @return
     */
    public boolean update(BC po){
        boolean bOk = this.updateById((T_BCImpl)po);
        return bOk;
    }

    /**
     * 删除数据
     * @param key 业务数据主键
     * @return
     */
    public boolean remove(String key){
        return this.removeById(key);
    }

    /**
     * 批量新建
     * @param entityList 需要新建的数据集合
     * @param batchSize 每次新建数量
     * @return
     */
    public void createBatch(List entityList, int batchSize) {
          this.saveBatch(entityList,batchSize);
    }

    /**
     * 批量更新
     * @param entityList 需要更新的数据集合
     * @param batchSize 每次更新数量
     * @return
     */
    public void updateBatch(List entityList, int batchSize) {
          this.updateBatchById(entityList,batchSize);
    }

    /**
     * 批量删除
     * @param idList 需要删除的数据主键集合
     * @return
     */
    public void removeBatch(Collection<? extends Serializable> idList) {
          this.removeByIds(idList);
    }

    /**
     * 批量建立数据，忽略建立数据过程中出现的错误
     * @param entityList 批量建立的数据集合
     * @return 建立过程中出现错误的数据主键集合
     */
    public JSONArray createBatchIgnoreErrors(Collection entityList) {

        Field keyField=null;
        JSONArray errorList = new JSONArray();
        Iterator it=entityList.iterator();
        while(it.hasNext()){
            Object obj = it.next();
            if(ObjectUtils.isEmpty(keyField)){
                keyField=deHelper.getKeyField(obj);
            }
            try {
                boolean isSuccess=this.create((T_BCImpl) obj);
                if(!isSuccess){
                    setErrorList(errorList,obj,keyField,"");
                }
            } catch (Exception e) {
                setErrorList(errorList,obj,keyField,e.getMessage());
            }
        }
        return errorList;
    }

    /**
     * 批量更新数据，忽略更新数据过程中出现的错误
     * @param entityList 批量更新的数据集合
     * @return 更新过程中出现错误的数据主键集合
     */
    public JSONArray updateBatchIgnoreErrors(Collection entityList) {

        Field keyField=null;
        JSONArray errorList = new JSONArray();
        Iterator it=entityList.iterator();
        while(it.hasNext()){
            Object obj = it.next();
            if(ObjectUtils.isEmpty(keyField)){
                keyField=deHelper.getKeyField(obj);
            }
            try {
                boolean isSuccess=this.update((T_BCImpl) obj);
                if(!isSuccess){
                    setErrorList(errorList,obj,keyField,"");
                }
            } catch (Exception e) {
                    setErrorList(errorList,obj,keyField,e.getMessage());
            }
        }
        return errorList;
    }

    /**
     * 批量删除数据，忽略删除数据过程中出现的错误
     * @param idList 批量删除的数据主键集合
     * @return 删除过程中出现错误的数据主键集合
     */
    public JSONArray removeBatchIgnoreErrors(Collection<? extends Serializable> idList) {

        JSONArray errorList = new JSONArray();
        Iterator it=idList.iterator();
        while(it.hasNext()){
            Object id = it.next();
            try {
                boolean isSuccess=this.remove((String)id);
                if(!isSuccess){
                    errorList.add(id);
                    logger.error(String.format("批量删除[%s]发生错误，出错的数据[%s]，错误原因[%s]","T_BC",id,""));
                }
            } catch (Exception e) {
                errorList.add(id);
                logger.error(String.format("批量删除[%s]发生错误，出错的数据[%s]，错误原因[%s]","T_BC",id,e));
            }
        }
        return errorList;
    }

    /**
     * 设置错误信息
     * @param errorList
     * @param obj
     * @param keyField
     * @param errorInfo
     */
    private void setErrorList(JSONArray errorList,Object obj, Field keyField,String errorInfo){

        Object keyFieldValue=deHelper.getKeyFieldValue(obj,keyField);
        if(!ObjectUtils.isEmpty(keyFieldValue)){
            errorList.add(keyFieldValue);
        }
        logger.error(String.format("操作[%s]发生错误，出错的数据[%s]，错误原因[%s]","T_BC",keyFieldValue,errorInfo));
    }

    /**
     * 通过条件更新数据
     * @param po 更新数据内容
     * @param context 更新条件
     * @return
     */
    public boolean updateByCond(BC po, BCSearchContext context) {
        QueryWrapper qw=queryBuilder.buildQueryWrapper(context);
        return this.update((T_BCImpl) po,qw);
    }

    /**
     * 通过条件删除数据
     * @param context 更新条件
     * @return
     */
    public boolean removeByCond(BCSearchContext context) {
        QueryWrapper qw=queryBuilder.buildQueryWrapper(context);
        return this.remove(qw);
    }

    /**
     * 查询集合[Default]
     * @param context 查询上下文对象
     * @return
     */
    public Page<BC> searchDefault(BCSearchContext context){
        QueryWrapper qw=queryBuilder.buildQueryWrapper(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page page=queryBuilder.buildPage(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<BC> page_datas=t_BCMapper.searchDefault(page,context,qw);
        List<BC> list_datas=queryBuilder.page2List(page_datas);
        return new PageImpl<BC>(list_datas, context.getPageable(), page_datas.getTotal());
    }

    /**
     * 持久层对象转业务对象
     * @param po 持久层对象
     * @param domain 业务对象
     * @return
     */
    public com.ibiz.core.ht.domain.BC convert2Domain( BC po ,com.ibiz.core.ht.domain.BC domain){
        if(po == null)
            return domain ;
        if(domain == null){
            domain = new com.ibiz.core.ht.domain.BC();
        }
        //if(po.getCreateDateDirtyFlag())
        domain.setCreateDate(po.getCreateDate());
        //if(po.getBzDirtyFlag())
        domain.setBz(po.getBz());
        //if(po.getUpdateDateDirtyFlag())
        domain.setUpdateDate(po.getUpdateDate());
        //if(po.getBmldidsDirtyFlag())
        domain.setBmldids(po.getBmldids());
        //if(po.getUpdateManDirtyFlag())
        domain.setUpdateMan(po.getUpdateMan());
        //if(po.getCreateManDirtyFlag())
        domain.setCreateMan(po.getCreateMan());
        //if(po.getWfinstanceidDirtyFlag())
        domain.setWfinstanceid(po.getWfinstanceid());
        //if(po.getWfstepDirtyFlag())
        domain.setWfstep(po.getWfstep());
        //if(po.getBCNameDirtyFlag())
        domain.setBCName(po.getBCName());
        //if(po.getBcwfstateDirtyFlag())
        domain.setBcwfstate(po.getBcwfstate());
        //if(po.getBCIdDirtyFlag())
        domain.setBCId(po.getBCId());
        //if(po.getOrgDeptNameDirtyFlag())
        domain.setOrgDeptName(po.getOrgDeptName());
        //if(po.getIMIdDirtyFlag())
        domain.setIMId(po.getIMId());
        //if(po.getOrgDeptIdDirtyFlag())
        domain.setOrgDeptId(po.getOrgDeptId());
        return domain ;
    }

    /**
     * 业务对象转持久层对象
     * @param domain 业务对象
     * @param po 持久层对象
     * @return
     */
    public BC convert2PO(com.ibiz.core.ht.domain.BC domain , BC po){
        if(domain == null)
            return po ;
        if(po==null){
            po = createPO() ;
        }
        if(domain.getCreateDateDirtyFlag())
            po.setCreateDate(domain.getCreateDate());
        if(domain.getBzDirtyFlag())
            po.setBz(domain.getBz());
        if(domain.getUpdateDateDirtyFlag())
            po.setUpdateDate(domain.getUpdateDate());
        if(domain.getBmldidsDirtyFlag())
            po.setBmldids(domain.getBmldids());
        if(domain.getUpdateManDirtyFlag())
            po.setUpdateMan(domain.getUpdateMan());
        if(domain.getCreateManDirtyFlag())
            po.setCreateMan(domain.getCreateMan());
        if(domain.getWfinstanceidDirtyFlag())
            po.setWfinstanceid(domain.getWfinstanceid());
        if(domain.getWfstepDirtyFlag())
            po.setWfstep(domain.getWfstep());
        if(domain.getBCNameDirtyFlag())
            po.setBCName(domain.getBCName());
        if(domain.getBcwfstateDirtyFlag())
            po.setBcwfstate(domain.getBcwfstate());
        if(domain.getBCIdDirtyFlag())
            po.setBCId(domain.getBCId());
        if(domain.getOrgDeptNameDirtyFlag())
            po.setOrgDeptName(domain.getOrgDeptName());
        if(domain.getIMIdDirtyFlag())
            po.setIMId(domain.getIMId());
        if(domain.getOrgDeptIdDirtyFlag())
            po.setOrgDeptId(domain.getOrgDeptId());

        return po ;
    }
}
