package com.ibiz.mybatis.ht;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Collection;
import java.io.Serializable;
import java.lang.reflect.Field;

import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONArray;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Repository;
import org.springframework.util.ObjectUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ibiz.core.repository.SRFFILERepository;
import com.ibiz.core.repository.po.SRFFILE;
import com.ibiz.core.ht.filter.SRFFILESearchContext;
import com.ibiz.mybatis.ht.mapper.T_SRFFILEMapper;
import com.ibiz.mybatis.ht.po.T_SRFFILEImpl;
import com.ibiz.mybatis.util.helper.QueryBuilder;
import com.ibiz.mybatis.util.helper.DEHelper;

/**
 * 关系型数据实体表[T_SRFFILE]存储对象
 */
@Slf4j
@Repository
@ConditionalOnProperty(prefix = "srffile", value="store" , havingValue = "sql", matchIfMissing = true)
public class T_SRFFILERepositoryImpl extends ServiceImpl<T_SRFFILEMapper, T_SRFFILEImpl> implements SRFFILERepository {

    private Log logger= LogFactory.getLog(T_SRFFILERepositoryImpl.class);


    /**
     * mybatis条件构造器
     */
    @Autowired
    QueryBuilder queryBuilder;
    /**
     * [T_SRFFILE]持久层映射对象
     */
    @Autowired
	T_SRFFILEMapper t_SRFFILEMapper;

    /**
     * 实体辅助对象
     */
    @Autowired
    DEHelper deHelper;

    /**
     * 创建持久层对象
     * @return
     */
    public SRFFILE createPO(){
        return new T_SRFFILEImpl();
    }

    /**
     * 获取数据
     * @param key 业务数据主键
     * @return
     */
    public SRFFILE get(String key){
        return this.getById(key);
    }

    /**
     * 新建数据
     * @param po 持久层数据对象
     * @return
     */
    public boolean create(SRFFILE po){
        boolean bOk = this.save((T_SRFFILEImpl)po);
        return bOk;
    }

    /**
     * 更新数据
     * @param po 持久层数据对象
     * @return
     */
    public boolean update(SRFFILE po){
        boolean bOk = this.updateById((T_SRFFILEImpl)po);
        return bOk;
    }

    /**
     * 删除数据
     * @param key 业务数据主键
     * @return
     */
    public boolean remove(String key){
        return this.removeById(key);
    }

    /**
     * 批量新建
     * @param entityList 需要新建的数据集合
     * @param batchSize 每次新建数量
     * @return
     */
    public void createBatch(List entityList, int batchSize) {
          this.saveBatch(entityList,batchSize);
    }

    /**
     * 批量更新
     * @param entityList 需要更新的数据集合
     * @param batchSize 每次更新数量
     * @return
     */
    public void updateBatch(List entityList, int batchSize) {
          this.updateBatchById(entityList,batchSize);
    }

    /**
     * 批量删除
     * @param idList 需要删除的数据主键集合
     * @return
     */
    public void removeBatch(Collection<? extends Serializable> idList) {
          this.removeByIds(idList);
    }

    /**
     * 批量建立数据，忽略建立数据过程中出现的错误
     * @param entityList 批量建立的数据集合
     * @return 建立过程中出现错误的数据主键集合
     */
    public JSONArray createBatchIgnoreErrors(Collection entityList) {

        Field keyField=null;
        JSONArray errorList = new JSONArray();
        Iterator it=entityList.iterator();
        while(it.hasNext()){
            Object obj = it.next();
            if(ObjectUtils.isEmpty(keyField)){
                keyField=deHelper.getKeyField(obj);
            }
            try {
                boolean isSuccess=this.create((T_SRFFILEImpl) obj);
                if(!isSuccess){
                    setErrorList(errorList,obj,keyField,"");
                }
            } catch (Exception e) {
                setErrorList(errorList,obj,keyField,e.getMessage());
            }
        }
        return errorList;
    }

    /**
     * 批量更新数据，忽略更新数据过程中出现的错误
     * @param entityList 批量更新的数据集合
     * @return 更新过程中出现错误的数据主键集合
     */
    public JSONArray updateBatchIgnoreErrors(Collection entityList) {

        Field keyField=null;
        JSONArray errorList = new JSONArray();
        Iterator it=entityList.iterator();
        while(it.hasNext()){
            Object obj = it.next();
            if(ObjectUtils.isEmpty(keyField)){
                keyField=deHelper.getKeyField(obj);
            }
            try {
                boolean isSuccess=this.update((T_SRFFILEImpl) obj);
                if(!isSuccess){
                    setErrorList(errorList,obj,keyField,"");
                }
            } catch (Exception e) {
                    setErrorList(errorList,obj,keyField,e.getMessage());
            }
        }
        return errorList;
    }

    /**
     * 批量删除数据，忽略删除数据过程中出现的错误
     * @param idList 批量删除的数据主键集合
     * @return 删除过程中出现错误的数据主键集合
     */
    public JSONArray removeBatchIgnoreErrors(Collection<? extends Serializable> idList) {

        JSONArray errorList = new JSONArray();
        Iterator it=idList.iterator();
        while(it.hasNext()){
            Object id = it.next();
            try {
                boolean isSuccess=this.remove((String)id);
                if(!isSuccess){
                    errorList.add(id);
                    logger.error(String.format("批量删除[%s]发生错误，出错的数据[%s]，错误原因[%s]","T_SRFFILE",id,""));
                }
            } catch (Exception e) {
                errorList.add(id);
                logger.error(String.format("批量删除[%s]发生错误，出错的数据[%s]，错误原因[%s]","T_SRFFILE",id,e));
            }
        }
        return errorList;
    }

    /**
     * 设置错误信息
     * @param errorList
     * @param obj
     * @param keyField
     * @param errorInfo
     */
    private void setErrorList(JSONArray errorList,Object obj, Field keyField,String errorInfo){

        Object keyFieldValue=deHelper.getKeyFieldValue(obj,keyField);
        if(!ObjectUtils.isEmpty(keyFieldValue)){
            errorList.add(keyFieldValue);
        }
        logger.error(String.format("操作[%s]发生错误，出错的数据[%s]，错误原因[%s]","T_SRFFILE",keyFieldValue,errorInfo));
    }

    /**
     * 通过条件更新数据
     * @param po 更新数据内容
     * @param context 更新条件
     * @return
     */
    public boolean updateByCond(SRFFILE po, SRFFILESearchContext context) {
        QueryWrapper qw=queryBuilder.buildQueryWrapper(context);
        return this.update((T_SRFFILEImpl) po,qw);
    }

    /**
     * 通过条件删除数据
     * @param context 更新条件
     * @return
     */
    public boolean removeByCond(SRFFILESearchContext context) {
        QueryWrapper qw=queryBuilder.buildQueryWrapper(context);
        return this.remove(qw);
    }

    /**
     * 查询集合[Default]
     * @param context 查询上下文对象
     * @return
     */
    public Page<SRFFILE> searchDefault(SRFFILESearchContext context){
        QueryWrapper qw=queryBuilder.buildQueryWrapper(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page page=queryBuilder.buildPage(context);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SRFFILE> page_datas=t_SRFFILEMapper.searchDefault(page,context,qw);
        List<SRFFILE> list_datas=queryBuilder.page2List(page_datas);
        return new PageImpl<SRFFILE>(list_datas, context.getPageable(), page_datas.getTotal());
    }

    /**
     * 持久层对象转业务对象
     * @param po 持久层对象
     * @param domain 业务对象
     * @return
     */
    public com.ibiz.core.ht.domain.SRFFILE convert2Domain( SRFFILE po ,com.ibiz.core.ht.domain.SRFFILE domain){
        if(po == null)
            return domain ;
        if(domain == null){
            domain = new com.ibiz.core.ht.domain.SRFFILE();
        }
        //if(po.getFilepathDirtyFlag())
        domain.setFilepath(po.getFilepath());
        //if(po.getUpdateManDirtyFlag())
        domain.setUpdateMan(po.getUpdateMan());
        //if(po.getOwneridDirtyFlag())
        domain.setOwnerid(po.getOwnerid());
        //if(po.getDigestcodeDirtyFlag())
        domain.setDigestcode(po.getDigestcode());
        //if(po.getFolderDirtyFlag())
        domain.setFolder(po.getFolder());
        //if(po.getPicheightDirtyFlag())
        domain.setPicheight(po.getPicheight());
        //if(po.getSRFFILENameDirtyFlag())
        domain.setSRFFILEName(po.getSRFFILEName());
        //if(po.getCreateDateDirtyFlag())
        domain.setCreateDate(po.getCreateDate());
        //if(po.getUpdateDateDirtyFlag())
        domain.setUpdateDate(po.getUpdateDate());
        //if(po.getPicwidthDirtyFlag())
        domain.setPicwidth(po.getPicwidth());
        //if(po.getSRFFILEIdDirtyFlag())
        domain.setSRFFILEId(po.getSRFFILEId());
        //if(po.getCreateManDirtyFlag())
        domain.setCreateMan(po.getCreateMan());
        //if(po.getFileextDirtyFlag())
        domain.setFileext(po.getFileext());
        //if(po.getFilesizeDirtyFlag())
        domain.setFilesize(po.getFilesize());
        return domain ;
    }

    /**
     * 业务对象转持久层对象
     * @param domain 业务对象
     * @param po 持久层对象
     * @return
     */
    public SRFFILE convert2PO(com.ibiz.core.ht.domain.SRFFILE domain , SRFFILE po){
        if(domain == null)
            return po ;
        if(po==null){
            po = createPO() ;
        }
        if(domain.getFilepathDirtyFlag())
            po.setFilepath(domain.getFilepath());
        if(domain.getUpdateManDirtyFlag())
            po.setUpdateMan(domain.getUpdateMan());
        if(domain.getOwneridDirtyFlag())
            po.setOwnerid(domain.getOwnerid());
        if(domain.getDigestcodeDirtyFlag())
            po.setDigestcode(domain.getDigestcode());
        if(domain.getFolderDirtyFlag())
            po.setFolder(domain.getFolder());
        if(domain.getPicheightDirtyFlag())
            po.setPicheight(domain.getPicheight());
        if(domain.getSRFFILENameDirtyFlag())
            po.setSRFFILEName(domain.getSRFFILEName());
        if(domain.getCreateDateDirtyFlag())
            po.setCreateDate(domain.getCreateDate());
        if(domain.getUpdateDateDirtyFlag())
            po.setUpdateDate(domain.getUpdateDate());
        if(domain.getPicwidthDirtyFlag())
            po.setPicwidth(domain.getPicwidth());
        if(domain.getSRFFILEIdDirtyFlag())
            po.setSRFFILEId(domain.getSRFFILEId());
        if(domain.getCreateManDirtyFlag())
            po.setCreateMan(domain.getCreateMan());
        if(domain.getFileextDirtyFlag())
            po.setFileext(domain.getFileext());
        if(domain.getFilesizeDirtyFlag())
            po.setFilesize(domain.getFilesize());

        return po ;
    }
}
